/*
**> Call/Put on himalaya
** 
** Let S1(t),...,Sn(t) be the vector of n assets
** let t1,...,tm be the vector of fixing dates
** At fixing date t1, we choose the best performer. We denote it B1.
** At fixing date t2, we choose the best performer excluding B1. We denote it B2.
** At fixing date t3, we choose the best performer excluding B1 and B2. We denote it B3.
** etc...
**
** Let B(T) = w1*B1(t1) + ... + wm*Bm(tm)
** The payoff of the Call option is: max(B(T) - K)
**
** Remarks: There exists many variants of the previous himalaya structure.
*/

new;
library option,pgraph;

cls;

let S0 = 100 100 100 100;            
let mu = 0.05 0.06 0.07 0.08;
let sigma = 0.15 0.25 0.20 0.30;                  // Implied Volatility

// Correlation Matrix between Asset returns

let rho = {1,
           0.5, 1,
           0.25, 0.15, 1,
           0.5,  0.5, 0.00, 1.00};
rho = xpnd(rho); 

// First, we have to correct the drift
r = 0.05;                                    // risk-free interest rate
mu = ones(rows(mu),1) * r;

// Caracteristics of the option
let t = 1 2 3;                               // 3 Fixing Dates: 1Y, 2Y, 3Y
tau = t[3];                                  // Maturity of the option = 3 Years
Notional = 1000;                             // Notional of the option
K = 0.05;                                    // Strike of the option = 3%
let w = 0.60 0.30 0.10;                      // Weights of the himalaya structure
fn call_payoff(S,K) = (S - K) .* (S .>= K);  // call option
fn put_payoff(S,K) = (K - S) .* (S .<= K);   // put option

Ns = 100000;  // Number of simulations
S = array_simulate_mGBM(S0,mu,sigma,rho,t,Ns);

Nt = rows(t);
B = zeros(Nt,Ns);
Exclusion_Index = zeros(3,1);

i = 1;
do until i > Ns;
  Stilde = mGBM_getSimul(S,i)' ./ S0 - 1;
  j = 1;
  do until j > 3;
    if j == 1;
      indx = rev(sortind(Stilde[.,j]));
      Exclusion_Index[1] = indx[1];
    elseif j == 2;
      Stilde[Exclusion_Index[1],j] = error(0);
      indx = rev(sortind(Stilde[.,j]));
      Exclusion_Index[2] = indx[1];
    elseif j == 3;
      Stilde[Exclusion_Index[1],j] = error(0);
      Stilde[Exclusion_Index[2],j] = error(0);
      indx = rev(sortind(Stilde[.,j]));
      Exclusion_Index[3] = indx[1];
    endif;                              
    B[j,i] = Stilde[indx[1],j];               
    j = j + 1;
  endo;
  i = i + 1;
endo;

Himalaya = sumc(w .* B);

C = Notional * exp(-r*tau) * meanc( call_payoff(Himalaya,K) );
P = Notional * exp(-r*tau) * meanc( put_payoff(Himalaya,K) );

print ftos(C,"Call Himalaya Option Price = %lf",3,2);
print ftos(P,"Put Himalaya Option Price = %lf",3,2);